require 'spec_helper'

describe 'simp::yum::repo::local_os_updates' do

  on_supported_os.each do |os, os_facts|
    context "on #{os}" do
      let(:facts) { os_facts }

      if os_facts[:kernel] == 'windows'
        it { expect{ is_expected.to compile.with_all_deps }.to raise_error(/'windows .+' is not supported/) }
        next
      end

      context 'with a single server name' do
        let(:params) {{ :servers => ['puppet.example.simp'] }}
        let(:os_name){ facts[:os][:name] }
        let(:os_maj_rel){ facts[:os][:release][:major] }

        it { is_expected.to compile.with_all_deps }
        it {
          os_yum_path =  "#{os_name}/#{os_maj_rel}/#{facts[:architecture]}"

          if os_name  == 'RedHat'
            gpgkey = "https://puppet.example.simp/yum/#{os_yum_path}/RPM-GPG-KEY-redhat-release"
          elsif os_name  == 'OracleLinux'
            gpgkey = "https://puppet.example.simp/yum/#{os_yum_path}/RPM-GPG-KEY-oracle"
          elsif  os_name == 'CentOS' and os_maj_rel  >= '8'
            gpgkey = "https://puppet.example.simp/yum/#{os_yum_path}/RPM-GPG-KEY-#{os_name}-Official"
          else
            gpgkey = "https://puppet.example.simp/yum/#{os_yum_path}/RPM-GPG-KEY-#{os_name}-#{os_maj_rel}"
          end

          is_expected.to contain_yumrepo('os_updates').with(
            :baseurl => "https://puppet.example.simp/yum/#{os_yum_path}/Updates",
            :gpgkey  => gpgkey
          )
        }

        context 'with relative_repo_path = x/y/z' do
          let(:params){super().merge( relative_repo_path: 'x/y/z')}
          it { is_expected.to compile.with_all_deps }
          it {
            if os_name  == 'RedHat'
              gpgkey = "https://puppet.example.simp/yum/x/y/z/RPM-GPG-KEY-redhat-release"
            elsif os_name  == 'OracleLinux'
              gpgkey = "https://puppet.example.simp/yum/x/y/z/RPM-GPG-KEY-oracle"
            elsif  os_name == 'CentOS' and os_maj_rel  >= '8'
              gpgkey = "https://puppet.example.simp/yum/x/y/z/RPM-GPG-KEY-#{os_name}-Official"
            else
              gpgkey = "https://puppet.example.simp/yum/x/y/z/RPM-GPG-KEY-#{os_name}-#{os_maj_rel}"
            end

            is_expected.to contain_yumrepo('os_updates').with(
              :baseurl => "https://puppet.example.simp/yum/x/y/z/Updates",
              :gpgkey  => gpgkey
            )
          }
        end
      end

      context 'with multiple servers and extra gpgkey URLs' do
        let(:params) {
          arbitrary_url = 'https://yum.test.simp:4433/repos/' +
                          "#{facts[:os][:name]}_#{facts[:os][:release][:major]}" +
                          "_#{facts[:architecture]}"
          {
          :servers => [
            'puppet.example.simp',
            '192.0.2.5',
            arbitrary_url,
          ],
          :extra_gpgkey_urls => [
            "#{arbitrary_url}/RPM-GPG-KEY-#{facts[:os][:name]}-#{facts[:os][:release][:major]}"
          ]
        }}

        it { is_expected.to compile.with_all_deps }
        it {
          os_maj_rel  = facts[:os][:release][:major]
          os_name     = facts[:os][:name]
          os_yum_path =  "#{os_name}/#{os_maj_rel}/#{facts[:architecture]}"
          arbitrary_url = 'https://yum.test.simp:4433/repos/' +
                          "#{facts[:os][:name]}_#{facts[:os][:release][:major]}" +
                          "_#{facts[:architecture]}"

          gpg_prefixes = ['puppet.example.simp', '192.0.2.5']
            .map{|x| "https://#{x}/yum/#{os_yum_path}" }

          if os_name  == 'RedHat'
            gpgkey = gpg_prefixes.map{|x| "#{x}/RPM-GPG-KEY-redhat-release" }.join("\n    ")
          elsif os_name  == 'OracleLinux'
            gpgkey = gpg_prefixes.map{|x| "#{x}/RPM-GPG-KEY-oracle" }.join("\n    ")
          elsif os_name == 'CentOS' and  os_maj_rel >= '8'
            gpgkey = gpg_prefixes.map{|x| "#{x}/RPM-GPG-KEY-#{os_name}-Official" }.join("\n    ")
          else
            gpgkey = gpg_prefixes.map{|x| "#{x}/RPM-GPG-KEY-#{os_name}-#{os_maj_rel}" }.join("\n    ")
          end
          gpgkey += "\n    #{arbitrary_url}/RPM-GPG-KEY-#{os_name}-#{os_maj_rel}"

          is_expected.to contain_yumrepo('os_updates').with(
            :baseurl => "https://puppet.example.simp/yum/#{os_yum_path}/Updates\n    " +
                        "https://192.0.2.5/yum/#{os_yum_path}/Updates\n    " +
                        arbitrary_url,
            :gpgkey  => gpgkey
          )
        }
      end

      context 'with baseurl and gpgkey overrides' do
        # No matter what OS we're testing, setting 'baseurl' and 'gpgkey'
        # directly should result in exactly the string that was specified
        # (in this case, repos for CentOS 8).
        let(:params) {{
            servers: ['puppet.example.simp'],
            baseurl: 'https://yum.test1.simp/yum/CentOS/8/x86_64/Updates',
            gpgkey:  'https://yum.test2.simp/yum/SIMP/GPGKEYS/RPM-GPG-KEY-CentOS-8',
        }}

        it { is_expected.to compile.with_all_deps }
        it {
          is_expected.to contain_yumrepo('os_updates').with(
            :baseurl => 'https://yum.test1.simp/yum/CentOS/8/x86_64/Updates',
            :gpgkey  => 'https://yum.test2.simp/yum/SIMP/GPGKEYS/RPM-GPG-KEY-CentOS-8',
          )
        }
      end
    end
  end
end
